/**
 * Copyright 2024 cronn GmbH
 * SPDX-License-Identifier: AGPL-3.0-only
 */

"use client";

import { ApiUserRole } from "@eshg/employee-portal-api/base";
import { ApiChatFeature } from "@eshg/employee-portal-api/chatManagement";
import { RequiresChildren } from "@eshg/lib-portal/types/react";
import { createContext, useContext, useMemo } from "react";
import { isNullish, omit } from "remeda";

import { useGetSelfUser } from "@/lib/baseModule/api/queries/users";
import { useMessagesSidebar } from "@/lib/baseModule/components/layout/messagesSidebar/MessagesSidebar";
import { useIsNewFeatureEnabledUnsuspended } from "@/lib/businessModules/chat/api/queries/featureTogglesApi";
import { useGetUserSettings } from "@/lib/businessModules/chat/api/queries/userSettingsApi";
import { ChatClientProvider } from "@/lib/businessModules/chat/shared/ChatClientProvider";
import { ChatConfiguration } from "@/lib/businessModules/chat/shared/config";
import { MessageTeaserProvider } from "@/lib/shared/components/chat/MessageTeaserProvider";
import { useHasUserRoleCheck } from "@/lib/shared/hooks/useAccessControl";
import { useIsOffline } from "@/lib/shared/hooks/useIsOffline";

import { ChatUserSettings } from "./types";

export interface ChatProviderContextType {
  configuration: ChatConfiguration;
  userSettings: ChatUserSettings;
  canAccessChat: boolean;
  isSettingsLoading: boolean;
  isFeatureToggleLoading: boolean;
  messagesSidebar: { isOpen: boolean; open: () => void; close: () => void };
}

const ChatContext = createContext<ChatProviderContextType | undefined>(
  undefined,
);

export interface ChatProviderProps extends RequiresChildren {
  configuration: ChatConfiguration;
}

export function ChatProvider(props: ChatProviderProps) {
  const isOffline = useIsOffline();

  return isOffline ? props.children : <InnerChatProvider {...props} />;
}

function InnerChatProvider({ children, configuration }: ChatProviderProps) {
  const {
    data: featureToggleChatEnabled,
    isLoading: featureToggleChatEnabledLoading,
  } = useIsNewFeatureEnabledUnsuspended(ApiChatFeature.ChatBase);

  const { data: selfUser } = useGetSelfUser();
  const canAccessChat =
    useHasUserRoleCheck(ApiUserRole.ChatManagementWrite) &&
    !!featureToggleChatEnabled &&
    selfUser.username !== "dummy"; //TODO: Don't allow dummy user to use chat until it becomes more stable in demo environment
  const messagesSidebar = useMessagesSidebar();
  const { data: userSettingsData, isLoading } = useGetUserSettings(
    selfUser.userId,
    canAccessChat,
  );

  // Chat user settings
  const userSettings = useMemo<ChatUserSettings>(
    () => ({
      chatConsentAsked: undefined,
      chatUsageEnabled: false,
      sharePresence: false,
      showReadConfirmation: false,
      showTypingNotification: false,
      ...(userSettingsData && omit(userSettingsData, ["userId"])),
    }),
    [userSettingsData],
  );

  return (
    <ChatContext.Provider
      value={{
        configuration,
        userSettings,
        canAccessChat,
        isSettingsLoading: isLoading,
        isFeatureToggleLoading: featureToggleChatEnabledLoading,
        messagesSidebar,
      }}
    >
      {userSettings.chatUsageEnabled ? (
        <MessageTeaserProvider>
          <ChatClientProvider>{children}</ChatClientProvider>
        </MessageTeaserProvider>
      ) : (
        children
      )}
    </ChatContext.Provider>
  );
}

export function useChat() {
  const context = useContext(ChatContext);

  if (isNullish(context)) {
    throw new Error("useChat was called outside ChatContextProvider");
  }

  return context;
}
