/**
 * Copyright 2024 cronn GmbH
 * SPDX-License-Identifier: AGPL-3.0-only
 */

import { ApiAppointmentBookingType } from "@eshg/employee-portal-api/stiProtection";
import { NumberField } from "@eshg/lib-portal/components/formFields/NumberField";
import { Button, Grid, Radio, Sheet, Stack, Typography } from "@mui/joy";
import { addMinutes, startOfHour } from "date-fns";
import { useFormikContext } from "formik";
import { useEffect, useId, useState } from "react";

import { useGetFreeAppointments } from "@/lib/businessModules/stiProtection/api/queries/appointmentBlocks";
import { Row } from "@/lib/shared/Row";
import { DateTimeField } from "@/lib/shared/components/formFields/DateTimeField";
import { RadioGroupField } from "@/lib/shared/components/formFields/RadioGroupField";
import { validateTodayOrFutureDate } from "@/lib/shared/helpers/validators";

import { AddNewProcedureForm } from "./AddNewProcedureSidebar";
import { AppointmentPickerField } from "./AppointmentPickerField";

function ConnectedAppointmentPicker({
  name,
  active,
  initialMonth,
  concern,
}: {
  name: string;
  active?: boolean;
  initialMonth: Date | null;
  concern: AddNewProcedureForm["concern"];
}) {
  const now = new Date();
  const [currentMonth, setCurrentMonth] = useState(initialMonth ?? new Date());
  const freeAppointments = useGetFreeAppointments({
    concern: concern === "" ? undefined : concern,
    earliestDate: startOfHour(now),
  });
  const monthAppointments = freeAppointments.data ?? [];

  return (
    <AppointmentPickerField
      name={name}
      required={"Bitte ein Termin auswählen"}
      active={active}
      currentMonth={currentMonth}
      setCurrentMonth={setCurrentMonth}
      monthAppointments={monthAppointments}
    />
  );
}

export function AppointmentForm() {
  const appointmentBlockDescriptionlId = useId();
  const { values, setFieldValue } = useFormikContext<AddNewProcedureForm>();

  const blockSectionSelected =
    values.appointmentType === ApiAppointmentBookingType.AppointmentBlock;
  const customSectionSelected =
    values.appointmentType === ApiAppointmentBookingType.UserDefined;

  useEffect(() => {
    if (!values.blockAppointment) {
      return;
    }
    void setFieldValue(
      "appointmentType",
      ApiAppointmentBookingType.AppointmentBlock,
    );
  }, [values.blockAppointment, setFieldValue]);
  useEffect(() => {
    if (!values.customAppointmentDate && !values.customAppointmentDuration) {
      return;
    }
    void setFieldValue(
      "appointmentType",
      ApiAppointmentBookingType.UserDefined,
    );
  }, [
    values.customAppointmentDate,
    values.customAppointmentDuration,
    setFieldValue,
  ]);

  return (
    <RadioGroupField
      name="appointmentType"
      required="Bitte ein Termin anliegen"
    >
      <Stack gap={2}>
        <Sheet
          aria-current={blockSectionSelected}
          onClick={() =>
            setFieldValue(
              "appointmentType",
              ApiAppointmentBookingType.AppointmentBlock,
            )
          }
          aria-description="Termin aus Terminblock wählen"
        >
          <Grid container spacing={3} direction="row">
            <Grid>
              <Radio
                sx={{ flexBasis: "max-content" }}
                name="appointmentType"
                value={ApiAppointmentBookingType.AppointmentBlock}
              />
            </Grid>
            <Grid xxs={10}>
              <Stack>
                <Typography
                  id={appointmentBlockDescriptionlId}
                  style={{ marginBottom: "16px" }}
                >
                  Aus Terminblock
                </Typography>
                <Row justifyContent="center" flex={1}>
                  <ConnectedAppointmentPicker
                    name="blockAppointment"
                    active={blockSectionSelected}
                    concern={values.concern}
                    initialMonth={values.blockAppointment?.start ?? null}
                  />
                </Row>
              </Stack>
            </Grid>
          </Grid>
        </Sheet>
        <Sheet
          aria-current={customSectionSelected}
          onClick={() =>
            setFieldValue(
              "appointmentType",
              ApiAppointmentBookingType.UserDefined,
            )
          }
          aria-description="Frei wählbarer Zeitraum für den Termin"
        >
          <Row>
            <Radio
              id="appointmentTypeCustom"
              name="appointmentType"
              value={ApiAppointmentBookingType.UserDefined}
            />
            <Stack gap={1}>
              <DateTimeField
                allowEmpty={!customSectionSelected}
                label="Individueller Termin"
                name="customAppointmentDate"
                validate={validateTodayOrFutureDate}
                required={
                  customSectionSelected ? "Bitte ein Datum eingeben" : undefined
                }
              />
              <CustomAppointmentQuickButtons />
              <NumberField
                label="Termin Dauer in Min."
                name="customAppointmentDuration"
                required={
                  customSectionSelected ? "Bitte ein Dauer eingeben" : undefined
                }
              />
            </Stack>
          </Row>
        </Sheet>
      </Stack>
    </RadioGroupField>
  );
}

function CustomAppointmentQuickButtons() {
  const { setFieldValue } = useFormikContext<AddNewProcedureForm>();
  function setCustomAppointment(inMinutes: number) {
    const now = new Date();
    const roundTo = 5;
    const roundMinutes = now.getMinutes() % roundTo;
    const minutesToAdd =
      inMinutes > 0 ? inMinutes - roundMinutes : roundTo - roundMinutes;
    const customTime = addMinutes(now, minutesToAdd - now.getTimezoneOffset());
    void setFieldValue(
      "customAppointmentDate",
      customTime.toISOString().slice(0, 16),
    );
  }

  return (
    <Row mb={2} justifyContent="right">
      <Button
        title="Individueller Termin in den nächsten Minuten setzen"
        onClick={() => setCustomAppointment(0)}
        size="sm"
        variant="soft"
      >
        Jetzt
      </Button>
      <Button
        title="Individueller Termin in ca. 10 Minuten setzen"
        onClick={() => setCustomAppointment(10)}
        size="sm"
        variant="soft"
      >
        in 10m
      </Button>
      <Button
        title="Individueller Termin in ca. 20 Minuten setzen"
        onClick={() => setCustomAppointment(20)}
        size="sm"
        variant="soft"
      >
        in 20m
      </Button>
      <Button
        title="Individueller Termin in ca. 30 Minuten setzen"
        onClick={() => setCustomAppointment(30)}
        size="sm"
        variant="soft"
      >
        in 30m
      </Button>
    </Row>
  );
}
